import { trigger, style, animate, transition } from '@angular/animations';
import { moveItemInArray, DragDropModule } from '@angular/cdk/drag-drop';
import { ClickOutsideModule } from 'ng4-click-outside';
import { FormsModule } from '@angular/forms';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { DomSanitizer, BrowserModule } from '@angular/platform-browser';
import { Injectable, EventEmitter, SkipSelf, Host, HostListener, ElementRef, Directive, Component, Input, Output, ViewChild, ChangeDetectorRef, defineInjectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ClipboardService = /** @class */ (function () {
    function ClipboardService() {
        this.onPaste = new EventEmitter();
        this.onCopy = new EventEmitter();
    }
    /**
     * @private
     * @return {?}
     */
    ClipboardService.prototype.generateTempTextArea = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var txtArea = document.createElement('textarea');
        txtArea.id = 'txt';
        txtArea.style.position = 'fixed';
        txtArea.style.top = '0';
        txtArea.style.left = '0';
        txtArea.style.opacity = '0';
        document.body.appendChild(txtArea);
        return txtArea;
    };
    /**
     * @private
     * @param {?} data
     * @return {?}
     */
    ClipboardService.prototype.sanitize = /**
     * @private
     * @param {?} data
     * @return {?}
     */
    function (data) {
        return data.replace(/^\t*(.+)\t*$/, "$1");
    };
    /**
     * @private
     * @param {?} data
     * @return {?}
     */
    ClipboardService.prototype.get2DArray = /**
     * @private
     * @param {?} data
     * @return {?}
     */
    function (data) {
        /** @type {?} */
        var array = data.split(/\r?\n/);
        /** @type {?} */
        var array2D = [[]];
        for (var i = 0; i < array.length; ++i) {
            /** @type {?} */
            var row = array[i].split("\t");
            if (row.length === 1 && row.indexOf("") >= 0)
                continue;
            array2D[i] = [].concat(row);
        }
        return array2D;
    };
    /**
     * @param {?} data
     * @return {?}
     */
    ClipboardService.prototype.copyToClipboard = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        /** @type {?} */
        var clipboardData = "";
        for (var i = 0; i < data.length; ++i) {
            if (!data[i]) {
                continue;
            }
            /** @type {?} */
            var row = data[i].join("\t");
            if (row === "") {
                continue;
            }
            clipboardData += this.sanitize(row) + '\n';
        }
        /** @type {?} */
        var txtArea;
        if (!this.txtAreaG) {
            this.txtAreaG = this.generateTempTextArea();
        }
        txtArea = this.txtAreaG;
        txtArea.value = clipboardData;
        txtArea.select();
        try {
            /** @type {?} */
            var successful = document.execCommand('copy');
            if (successful) {
                this.onCopy.emit(true);
                return true;
            }
        }
        catch (err) {
            console.error('Oops, unable to copy');
        }
        finally {
            //document.body.removeChild(txtArea);
        }
        return false;
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.getClipboardData = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var txtArea;
        if (!this.txtAreaG) {
            this.txtAreaG = this.generateTempTextArea();
        }
        txtArea = this.txtAreaG;
        txtArea.focus();
        /** @type {?} */
        var result = '';
        try {
            if (result = txtArea.value) ;
        }
        catch (err) {
            console.error('Oops, unable to paste!');
        }
        finally {
            this.onPaste.emit(this.get2DArray(result));
            return this.get2DArray(result);
        }
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.getPasteEvent = /**
     * @return {?}
     */
    function () {
        return this.onPaste;
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.getCopyEvent = /**
     * @return {?}
     */
    function () {
        return this.onCopy;
    };
    ClipboardService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    ClipboardService.ctorParameters = function () { return []; };
    /** @nocollapse */ ClipboardService.ngInjectableDef = defineInjectable({ factory: function ClipboardService_Factory() { return new ClipboardService(); }, token: ClipboardService, providedIn: "root" });
    return ClipboardService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FilterService = /** @class */ (function () {
    function FilterService() {
    }
    /**
     * @private
     * @param {?} column
     * @param {?} filterOptions
     * @param {?} data
     * @return {?}
     */
    FilterService.prototype.getFilteredValue = /**
     * @private
     * @param {?} column
     * @param {?} filterOptions
     * @param {?} data
     * @return {?}
     */
    function (column, filterOptions, data) {
        /** @type {?} */
        var filtered;
        if (!filterOptions[column].values.length) {
            return true;
        }
        for (var i = 0; i < filterOptions[column].values.length; ++i) {
            if (filterOptions[column].operator == 'or') {
                filtered = !!filtered || filterOptions[column].comparator.call(this, data, filterOptions[column].values[i]);
            }
            if (filterOptions[column].operator == 'and') {
                filtered = !filtered && filterOptions[column].comparator.call(this, data, filterOptions[column].values[i]);
            }
        }
        return filtered;
    };
    /**
     * @param {?} data
     * @param {?} tableRows
     * @return {?}
     */
    FilterService.prototype.filterCommon = /**
     * @param {?} data
     * @param {?} tableRows
     * @return {?}
     */
    function (data, tableRows) {
        /** @type {?} */
        var FilterRowCount = 0;
        /** @type {?} */
        var result = { tableRows: [], FilteredRowCount: 0 };
        for (var i = 0; i < tableRows.length; ++i) {
            /** @type {?} */
            var isFiltered = void 0;
            for (var j = 0; j < data.length; ++j) {
                if (data[j] === undefined) {
                    //TODO: Change the INCLUDES function to the new UTILS service.
                    isFiltered = isFiltered || tableRows[i].data[j].toString().includes('');
                    continue;
                }
                isFiltered = isFiltered || this.getFilteredValue(j, data, tableRows[i].data[j].toString());
            }
            tableRows[i].filteredOut = !isFiltered;
            console.log("Row: " + i, "Filtered: " + isFiltered, "Filtered Out: " + tableRows[i].filteredOut, "Data: " + JSON.stringify(tableRows[i].data));
            if (!tableRows[i].filteredOut) {
                FilterRowCount++;
            }
        }
        result.FilteredRowCount = FilterRowCount;
        result.tableRows = tableRows;
        return result;
    };
    /**
     * @param {?} data
     * @param {?} tableRows
     * @return {?}
     */
    FilterService.prototype.filter = /**
     * @param {?} data
     * @param {?} tableRows
     * @return {?}
     */
    function (data, tableRows) {
        /** @type {?} */
        var FilterRowCount = 0;
        /** @type {?} */
        var result = { tableRows: [], FilteredRowCount: 0 };
        for (var i = 0; i < tableRows.length; ++i) {
            /** @type {?} */
            var isFiltered = true;
            for (var j = 0; j < data.length; ++j) {
                if (data[j] === undefined) {
                    //TODO: Change the INCLUDES function to the new UTILS service.
                    isFiltered = isFiltered && tableRows[i].data[j].toString().includes('');
                    continue;
                }
                isFiltered = isFiltered && this.getFilteredValue(j, data, tableRows[i].data[j].toString());
            }
            tableRows[i].filteredOut = !isFiltered;
            if (!tableRows[i].filteredOut) {
                FilterRowCount++;
            }
        }
        result.FilteredRowCount = FilterRowCount;
        result.tableRows = tableRows;
        return result;
    };
    FilterService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    FilterService.ctorParameters = function () { return []; };
    /** @nocollapse */ FilterService.ngInjectableDef = defineInjectable({ factory: function FilterService_Factory() { return new FilterService(); }, token: FilterService, providedIn: "root" });
    return FilterService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var StringUtilsService = /** @class */ (function () {
    function StringUtilsService() {
    }
    /**
     *
     * @param s
     * @param c
       */
    /**
     *
     * @param {?} s
     * @param {?} c
     * @return {?}
     */
    StringUtilsService.equals = /**
     *
     * @param {?} s
     * @param {?} c
     * @return {?}
     */
    function (s, c) {
        if (typeof (c) !== "string") {
            c = c.toString();
        }
        return s.toLowerCase() == c.toLowerCase();
    };
    /**
     * 4
     * @param s
     * @param c
     * @param p
     */
    /**
     * 4
     * @param {?} s
     * @param {?} searchString
     * @param {?} position
     * @return {?}
     */
    StringUtilsService.includes = /**
     * 4
     * @param {?} s
     * @param {?} searchString
     * @param {?} position
     * @return {?}
     */
    function (s, searchString, position) {
        /** @type {?} */
        var stringS = searchString;
        if (typeof (stringS) !== "string") {
            searchString = stringS.toString();
        }
        /** @type {?} */
        var k = s.toLowerCase().match(searchString.toLowerCase());
        if (position && k) {
            return k.index === position;
        }
        return !!k;
    };
    StringUtilsService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    StringUtilsService.ctorParameters = function () { return []; };
    /** @nocollapse */ StringUtilsService.ngInjectableDef = defineInjectable({ factory: function StringUtilsService_Factory() { return new StringUtilsService(); }, token: StringUtilsService, providedIn: "root" });
    return StringUtilsService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DataTableUtilsService = /** @class */ (function () {
    function DataTableUtilsService() {
        this.onDelete = new EventEmitter();
    }
    /**
     * @return {?}
     */
    DataTableUtilsService.prototype.getOnDeleteEvent = /**
     * @return {?}
     */
    function () {
        return this.onDelete;
    };
    /**
     * @param {?} contextData
     * @return {?}
     */
    DataTableUtilsService.prototype.delete = /**
     * @param {?} contextData
     * @return {?}
     */
    function (contextData) {
        this.onDelete.emit(contextData);
    };
    DataTableUtilsService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    DataTableUtilsService.ctorParameters = function () { return []; };
    /** @nocollapse */ DataTableUtilsService.ngInjectableDef = defineInjectable({ factory: function DataTableUtilsService_Factory() { return new DataTableUtilsService(); }, token: DataTableUtilsService, providedIn: "root" });
    return DataTableUtilsService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DataTableComponent = /** @class */ (function () {
    function DataTableComponent(clipboardService, filterService, dataTableService, ref) {
        this.clipboardService = clipboardService;
        this.filterService = filterService;
        this.dataTableService = dataTableService;
        this.ref = ref;
        this.rowSelection = true;
        this.commonSearch = false;
        this.isMoving = false;
        this.Moved = [];
        this.math = Math;
        this.TableRows = [];
        this.PagedRows = [];
        this.CurrentPage = 1;
        this.InvalidPage = 0;
        this.FromRecord = 1;
        this.ToRecord = 0;
        this.FilteredRows = [];
        this.contextmenuX = 0;
        this.contextmenuY = 0;
        this.filterMenuX = 0;
        this.filterMenuY = 0;
        this.contextMenuData = [];
        this.rowSizes = [25, 50, 75, 100];
        this.selectAllRows = false;
    }
    // Convert row data to a 2D array.
    // Convert row data to a 2D array.
    /**
     * @param {?=} filteredData
     * @param {?=} currentPage
     * @return {?}
     */
    DataTableComponent.prototype.createTableData = 
    // Convert row data to a 2D array.
    /**
     * @param {?=} filteredData
     * @param {?=} currentPage
     * @return {?}
     */
    function (filteredData, currentPage) {
        this.TableRows = new Array();
        this.contextMenuData = [];
        if (!(this.rowData && this.rowData.length)) {
            this.rowData = [];
        }
        else if (this.columnDefs.length !== Object.keys(this.rowData[0]).length) {
            console.warn('Invalid data: Total Column in def: ' + this.columnDefs.length + 'Total Columns in data:'
                + Object.keys(this.rowData[0]).length);
        }
        for (var j = 0; j < this.rowData.length; ++j) {
            /** @type {?} */
            var row = { data: [], filteredOut: false };
            for (var i = 0; i < this.columnDefs.length; ++i) {
                if (!(filteredData && filteredData.length !== 0 && currentPage > 0)) {
                    this.columnDefs[i].sortState = null;
                    this.columnDefs[i].showFilter = false;
                }
                row.cellRender = this.cellRenderer;
                row.data.push(this.rowData[j][this.columnDefs[i].field]);
                if (this.rowSelection) {
                    row.rowSelect = false;
                }
            }
            this.TableRows.push(row);
            this.FilteredRows.push(row);
        }
        this.generateUniqueFilters();
        if (filteredData && filteredData.length !== 0 && currentPage > 0) {
            this.applyFilter(this.FilterData, this.TableRows);
            this.setPagedRow(currentPage);
            for (var i = 0; i < this.columnDefs.length; ++i) {
                if (this.columnDefs[i].sortState !== null) {
                    this.applySort(i, this.columnDefs[i].sortState);
                }
            }
        }
        else {
            this.pagedRows();
            this.setPagedRow(1);
        }
    };
    /**
     * @private
     * @param {?} isSelect
     * @return {?}
     */
    DataTableComponent.prototype.allRows = /**
     * @private
     * @param {?} isSelect
     * @return {?}
     */
    function (isSelect) {
        for (var i = 0; i < this.TableRows.length; ++i) {
            this.TableRows[i].rowSelect = isSelect;
            if (isSelect) {
                if (!(this.contextMenuData[i] && this.contextMenuData[i].length)) {
                    this.contextMenuData[i] = [];
                }
                this.contextMenuData[i] = this.TableRows[i].data;
            }
        }
        if (!isSelect) {
            this.contextMenuData = [];
        }
    };
    /**
     * @private
     * @param {?} selected
     * @param {?} rowNumber
     * @param {?=} isAll
     * @return {?}
     */
    DataTableComponent.prototype.selectRows = /**
     * @private
     * @param {?} selected
     * @param {?} rowNumber
     * @param {?=} isAll
     * @return {?}
     */
    function (selected, rowNumber, isAll) {
        /** @type {?} */
        var row = this.PagedRows[rowNumber];
        if (isAll) {
            this.allRows(selected);
            return;
        }
        if (row.rowSelect && selected) {
            if (!(this.contextMenuData[rowNumber] && this.contextMenuData[rowNumber].length)) {
                this.contextMenuData[rowNumber] = [];
            }
            this.contextMenuData[rowNumber] = this.PagedRows[rowNumber].data;
        }
        else if (!selected) {
            this.contextMenuData[rowNumber] = [];
        }
    };
    /**
     * @param {?} event
     * @param {?} isHeader
     * @return {?}
     */
    DataTableComponent.prototype.checkedRowSelection = /**
     * @param {?} event
     * @param {?} isHeader
     * @return {?}
     */
    function (event, isHeader) {
        if (isHeader) {
            this.selectRows(event.target.checked, null, true);
            return;
        }
        if (!event.value) {
            this.selectAllRows = false;
        }
        this.PagedRows[event.row].rowSelect = event.value;
        this.selectRows(event.value, event.row);
    };
    /**
     * @private
     * @return {?}
     */
    DataTableComponent.prototype.generateUniqueFilters = /**
     * @private
     * @return {?}
     */
    function () {
        for (var i = 0; i < this.columnDefs.length; ++i) {
            this.createColumnFilter(this.columnDefs[i], this.TableRows, i);
        }
    };
    /**
     * @private
     * @param {?} column
     * @param {?} rows
     * @param {?} columnNumber
     * @return {?}
     */
    DataTableComponent.prototype.createColumnFilter = /**
     * @private
     * @param {?} column
     * @param {?} rows
     * @param {?} columnNumber
     * @return {?}
     */
    function (column, rows, columnNumber) {
        /** @type {?} */
        var uniqueItems = [];
        column.uniqueFilterValues = [];
        if (!column.columnFilter) {
            return;
        }
        for (var i = 0; i < rows.length; ++i) {
            if (rows[i].filteredOut) {
                continue;
            }
            /** @type {?} */
            var columnValue = rows[i].data[columnNumber];
            if (uniqueItems.indexOf(columnValue) < 0) {
                uniqueItems.push(columnValue);
                column.uniqueFilterValues.push({ checked: false, filteredOut: false, data: [columnValue] });
                if (!this.FilterData[columnNumber]) {
                    this.FilterData[columnNumber] = { comparator: StringUtilsService.includes, operator: 'or', values: [] };
                }
                this.FilterData[columnNumber].values.push(columnValue);
            }
        }
        //column.uniqueFilterValues = uniqueItems;
    };
    /**
     * @private
     * @return {?}
     */
    DataTableComponent.prototype.pagedRows = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var j = 0;
        for (var i = 0; i < this.TableRows.length; ++i) {
            /** @type {?} */
            var row = this.TableRows[i];
            if (row.filteredOut) {
                row.pageNo = 0;
                continue;
            }
            if (this.pagination) {
                row.pageNo = Math.ceil((j + 1) / this.pageSize);
            }
            else {
                row.pageNo = 1;
            }
            j++;
        }
    };
    /**
     * @private
     * @param {?} pageNo
     * @return {?}
     */
    DataTableComponent.prototype.setPagedRow = /**
     * @private
     * @param {?} pageNo
     * @return {?}
     */
    function (pageNo) {
        this.PagedRows = [];
        for (var j = 0; j < this.TableRows.length; ++j) {
            /** @type {?} */
            var row = this.TableRows[j];
            if (row.pageNo === this.InvalidPage) {
                continue;
            }
            if (row.pageNo === pageNo) {
                this.PagedRows.push(row);
            }
        }
    };
    /**
     * @private
     * @param {?} row
     * @param {?} column
     * @param {?} data
     * @param {?} columnDefs
     * @return {?}
     */
    DataTableComponent.prototype.cellRenderer = /**
     * @private
     * @param {?} row
     * @param {?} column
     * @param {?} data
     * @param {?} columnDefs
     * @return {?}
     */
    function (row, column, data, columnDefs) {
        if (columnDefs[column].cellRender === undefined && (typeof (columnDefs[column].cellRender) !== 'function')) {
            return data;
        }
        else {
            return columnDefs[column].cellRender(row, column, data, columnDefs);
        }
    };
    // Filters data based on CONTAINS.
    // Filters data based on CONTAINS.
    /**
     * @param {?} column
     * @param {?} text
     * @return {?}
     */
    DataTableComponent.prototype.filter = 
    // Filters data based on CONTAINS.
    /**
     * @param {?} column
     * @param {?} text
     * @return {?}
     */
    function (column, text) {
        this.FilterData[column] = { operator: 'or', values: [text], comparator: StringUtilsService.includes };
        this.applyFilter(this.FilterData, this.TableRows);
    };
    /**
     * @private
     * @param {?} column
     * @param {?} filterOptions
     * @param {?} data
     * @return {?}
     */
    DataTableComponent.prototype.getFilteredValue = /**
     * @private
     * @param {?} column
     * @param {?} filterOptions
     * @param {?} data
     * @return {?}
     */
    function (column, filterOptions, data) {
        /** @type {?} */
        var filtered = false;
        if (!filterOptions[column].values.length) {
            return true;
        }
        for (var i = 0; i < filterOptions[column].values.length; ++i) {
            if (filterOptions[column].operator == 'or') {
                filtered = filtered || filterOptions[column].comparator.call(data, filterOptions[column].values[i]);
            }
        }
        return filtered;
    };
    /**
     * Apply filter.
     * @param filterData
     * @param tableRows
     */
    /**
     * Apply filter.
     * @private
     * @param {?} filterData
     * @param {?} tableRows
     * @return {?}
     */
    DataTableComponent.prototype.applyFilter = /**
     * Apply filter.
     * @private
     * @param {?} filterData
     * @param {?} tableRows
     * @return {?}
     */
    function (filterData, tableRows) {
        /** @type {?} */
        var result = this.filterService.filter(filterData, tableRows);
        this.FilterRowCount = result.FilteredRowCount;
        tableRows = result.tableRows;
        this.pagedRows();
        this.setPagedRow(1);
        this.updateTotalPageCount();
        this.contextMenuData = [];
    };
    /**
     * Called If any Unique valiue checkbox use for filter the column.
     * @param filterEventArgs
     */
    /**
     * Called If any Unique valiue checkbox use for filter the column.
     * @param {?} filterEventArgs
     * @return {?}
     */
    DataTableComponent.prototype.checkedColumnFilter = /**
     * Called If any Unique valiue checkbox use for filter the column.
     * @param {?} filterEventArgs
     * @return {?}
     */
    function (filterEventArgs) {
        this.FilterData[filterEventArgs.column].values = [];
        this.FilterData[filterEventArgs.column].comparator = StringUtilsService.equals;
        for (var i = 0; i < filterEventArgs.filteredData.length; ++i) {
            this.FilterData[filterEventArgs.column].values.push(filterEventArgs.filteredData[i]);
        }
        this.applyFilter(this.FilterData, this.TableRows);
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.updateTotalPageCount = /**
     * @return {?}
     */
    function () {
        this.TotalPages = Math.ceil(this.FilterRowCount / this.pageSize);
        this.FromRecord = 1;
        this.TotalRows = this.FilterRowCount;
        if (this.FilterRowCount < this.pageSize) {
            this.ToRecord = this.FilterRowCount;
        }
        else {
            this.ToRecord = this.pageSize;
        }
    };
    // Nevigate to Next Page
    // Nevigate to Next Page
    /**
     * @return {?}
     */
    DataTableComponent.prototype.nextPage = 
    // Nevigate to Next Page
    /**
     * @return {?}
     */
    function () {
        this.CurrentPage++;
        this.FromRecord += this.pageSize;
        this.ToRecord += this.pageSize;
        if (this.ToRecord > this.rowData.length) {
            this.ToRecord = this.rowData.length;
        }
        this.setPagedRow(this.CurrentPage);
    };
    // Nevigate to Previous page
    // Nevigate to Previous page
    /**
     * @return {?}
     */
    DataTableComponent.prototype.previousPage = 
    // Nevigate to Previous page
    /**
     * @return {?}
     */
    function () {
        this.CurrentPage--;
        this.FromRecord -= this.pageSize;
        this.ToRecord -= this.pageSize;
        this.setPagedRow(this.CurrentPage);
    };
    // Nevigate to Last page
    // Nevigate to Last page
    /**
     * @return {?}
     */
    DataTableComponent.prototype.lastPage = 
    // Nevigate to Last page
    /**
     * @return {?}
     */
    function () {
        this.ToRecord = this.FilterRowCount;
        this.CurrentPage = this.TotalPages;
        this.setPagedRow(this.CurrentPage);
        this.FromRecord = this.FilterRowCount - this.PagedRows.length + 1;
    };
    // Nevigate to First page
    // Nevigate to First page
    /**
     * @return {?}
     */
    DataTableComponent.prototype.firstPage = 
    // Nevigate to First page
    /**
     * @return {?}
     */
    function () {
        this.FromRecord = 1;
        this.ToRecord = this.pageSize;
        this.CurrentPage = 1;
        this.setPagedRow(this.CurrentPage);
    };
    // Sort columns
    // Sort columns
    /**
     * @param {?} column
     * @return {?}
     */
    DataTableComponent.prototype.sortColumn = 
    // Sort columns
    /**
     * @param {?} column
     * @return {?}
     */
    function (column) {
        // check whether it is sortable.
        if (!this.columnDefs[column].sort) {
            return;
        }
        // Reset all other sort
        for (var i = 0; i < this.columnDefs.length; ++i) {
            if (i === column) {
                continue;
            }
            this.columnDefs[i].sortState = null;
        }
        // cache the sort state
        /** @type {?} */
        var sortState = this.columnDefs[column].sortState;
        if (sortState == null) {
            this.columnDefs[column].sortState = true;
        }
        else {
            this.columnDefs[column].sortState = !this.columnDefs[column].sortState;
        }
        sortState = this.columnDefs[column].sortState;
        this.applySort(column, sortState);
    };
    //
    //
    /**
     * @private
     * @param {?} column
     * @param {?} sortState
     * @return {?}
     */
    DataTableComponent.prototype.applySort = 
    //
    /**
     * @private
     * @param {?} column
     * @param {?} sortState
     * @return {?}
     */
    function (column, sortState) {
        /** @type {?} */
        var that = this;
        // Sort te table.
        this.TableRows.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        function (a, b) { return that.sortFunction(a, b, column, sortState); }));
        this.pagedRows();
        this.setPagedRow(this.CurrentPage);
        this.contextMenuData = [];
    };
    // Sort function
    // Sort function
    /**
     * @private
     * @param {?} a
     * @param {?} b
     * @param {?} columnValue
     * @param {?} isAsc
     * @return {?}
     */
    DataTableComponent.prototype.sortFunction = 
    // Sort function
    /**
     * @private
     * @param {?} a
     * @param {?} b
     * @param {?} columnValue
     * @param {?} isAsc
     * @return {?}
     */
    function (a, b, columnValue, isAsc) {
        if (a.data[columnValue] === b.data[columnValue]) {
            return 0;
        }
        else if (isAsc) {
            return (a.data[columnValue] < b.data[columnValue]) ? -1 : 1;
        }
        else {
            return (a.data[columnValue] > b.data[columnValue]) ? -1 : 1;
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.dragEnded = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.clientWidth = event.source.element.nativeElement.offsetWidth;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.drop = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.isMoving = false;
        this.previousIndex = undefined;
        moveItemInArray(this.columnDefs, event.previousIndex, event.currentIndex);
        moveItemInArray(this.FilterData, event.previousIndex, event.currentIndex);
        this.createTableData(this.FilterData, this.CurrentPage);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.dropRow = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        moveItemInArray(this.PagedRows, event.previousIndex, event.currentIndex);
        this.allRows(false);
        this.contextMenuData = [];
        this.selectAllRows = false;
    };
    /**
     * @param {?} changeValue
     * @return {?}
     */
    DataTableComponent.prototype.valueChanged = /**
     * @param {?} changeValue
     * @return {?}
     */
    function (changeValue) {
        this.TableRows[changeValue.row].data[changeValue.column] = changeValue.value;
        this.pagedRows();
        this.setPagedRow(this.CurrentPage);
    };
    /**
     * @private
     * @return {?}
     */
    DataTableComponent.prototype.selectedRowsCount = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var rowCount = 0;
        for (var i = 0; i < this.contextMenuData.length; ++i) {
            if (!this.contextMenuData[i])
                continue;
            if (!this.contextMenuData[i].length)
                continue;
            rowCount++;
        }
        return rowCount;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.showContextMenu = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.contextmenuX = event.x;
        this.contextmenuY = event.y;
        if (!(this.contextMenuData[event.row] && this.contextMenuData[event.row][event.column]) && event.column !== -1) {
            this.contextMenuData = [];
            if (!this.contextMenuData[event.row]) {
                this.contextMenuData[event.row] = [];
            }
            this.contextMenuData[event.row][event.column] = this.PagedRows[event.row].data[event.column];
        }
        this.contextMenuIsEdit = event.isEdit;
        this.contextmenu = true;
    };
    /**
     * On Key up common filter.
     * @param text: String to be searched across the table data.
     */
    /**
     * On Key up common filter.
     * @param {?} text
     * @return {?}
     */
    DataTableComponent.prototype.onCommonFilter = /**
     * On Key up common filter.
     * @param {?} text
     * @return {?}
     */
    function (text) {
        for (var i = 0; i < this.commonFilterData.length; ++i) {
            this.commonFilterData[i] = this.commonFilterData[i] || {
                comparator: StringUtilsService.includes,
                operator: 'and',
                values: []
            };
            this.commonFilterData[i].values = [text];
        }
        /** @type {?} */
        var result = this.filterService.filterCommon(this.commonFilterData, this.TableRows);
        this.FilterRowCount = result.FilteredRowCount;
        this.TableRows = result.tableRows;
        this.pagedRows();
        this.setPagedRow(1);
        this.updateTotalPageCount();
        this.contextMenuData = [];
    };
    /**
     * @param {?} column
     * @return {?}
     */
    DataTableComponent.prototype.hasData = /**
     * @param {?} column
     * @return {?}
     */
    function (column) {
        for (var i = 0; i < this.contextMenuData.length; ++i) {
            if (!this.contextMenuData[i]) {
                continue;
            }
            for (var j = 0; j < this.contextMenuData[i].length; ++i) {
                if (!this.contextMenuData[i][j]) {
                    return true;
                }
            }
        }
        return false;
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.unSelectRows = /**
     * @return {?}
     */
    function () {
        for (var i = 0; i < this.TableRows.length; ++i) {
            this.TableRows[i].rowSelect = false;
        }
    };
    /**
     * @param {?} rowCount
     * @param {?} columnCount
     * @param {?} cell
     * @param {?} $event
     * @return {?}
     */
    DataTableComponent.prototype.onSelecting = /**
     * @param {?} rowCount
     * @param {?} columnCount
     * @param {?} cell
     * @param {?} $event
     * @return {?}
     */
    function (rowCount, columnCount, cell, $event) {
        if (this.isDragging) {
            if (!this.contextMenuData[rowCount]) {
                this.contextMenuData[rowCount] = [];
                this.PagedRows[rowCount].rowSelect = false;
            }
            this.contextMenuData[rowCount][columnCount] = this.PagedRows[rowCount].data[columnCount];
            this.selectedRowsColumns();
        }
    };
    /**
     * @param {?} rowCount
     * @param {?} columnCount
     * @param {?} cell
     * @param {?} $event
     * @return {?}
     */
    DataTableComponent.prototype.onClick = /**
     * @param {?} rowCount
     * @param {?} columnCount
     * @param {?} cell
     * @param {?} $event
     * @return {?}
     */
    function (rowCount, columnCount, cell, $event) {
        if (this.contextMenuData[rowCount] && this.contextMenuData[rowCount][columnCount]) {
            this.contextMenuData = [];
            this.unSelectRows();
        }
        else {
            if (!$event.ctrlKey) {
                this.contextMenuData = [];
                this.unSelectRows();
            }
            if (!this.contextMenuData[rowCount]) {
                this.PagedRows[rowCount].rowSelect = false;
                this.contextMenuData[rowCount] = [];
            }
            this.contextMenuData[rowCount][columnCount] = this.PagedRows[rowCount].data[columnCount];
        }
    };
    /**
     * @param {?} row
     * @param {?} column
     * @param {?} $event
     * @return {?}
     */
    DataTableComponent.prototype.onDragStart = /**
     * @param {?} row
     * @param {?} column
     * @param {?} $event
     * @return {?}
     */
    function (row, column, $event) {
        //For left click only
        if ($event.button === 0) {
            this.contextMenuData = [];
            this.isDragging = true;
        }
    };
    /**
     * @private
     * @return {?}
     */
    DataTableComponent.prototype.selectedRowsColumns = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var selectionRowStart = null;
        /** @type {?} */
        var selectionRowEnd = null;
        /** @type {?} */
        var selectionColStart = null;
        /** @type {?} */
        var selectionColEnd = null;
        for (var i = 0; i < this.contextMenuData.length; ++i) {
            /** @type {?} */
            var col = this.contextMenuData[i];
            if (!(col && col.length)) {
                continue;
            }
            if (selectionColStart === null) {
                selectionRowEnd = i;
                selectionRowStart = i;
            }
            if (i > selectionRowEnd) {
                selectionRowEnd = i;
            }
            for (var j = 0; j < col.length; ++j) {
                if (!col[j]) {
                    continue;
                }
                if (selectionColStart === null) {
                    selectionColStart = j;
                    selectionColEnd = j;
                }
                if (j < selectionColStart) {
                    selectionColStart = j;
                }
                if (j > selectionColEnd) {
                    selectionColEnd = j;
                }
            }
        }
        if (selectionRowStart === null || selectionRowEnd === null
            || selectionColEnd === null || selectionColStart === null) {
            return;
        }
        for (var i = selectionRowStart; i <= selectionRowEnd; ++i) {
            for (var j = selectionColStart; j <= selectionColEnd; ++j) {
                if (!this.contextMenuData[i])
                    this.contextMenuData[i] = [];
                this.contextMenuData[i][j] = this.TableRows[i].data[j];
            }
        }
    };
    /**
     * @param {?} row
     * @param {?} column
     * @param {?} $event
     * @return {?}
     */
    DataTableComponent.prototype.onDragEnd = /**
     * @param {?} row
     * @param {?} column
     * @param {?} $event
     * @return {?}
     */
    function (row, column, $event) {
        //For left click only
        if ($event.button === 0) {
            this.selectedRowsColumns();
            this.isDragging = false;
        }
    };
    /**
     * @private
     * @param {?} currentIndex
     * @param {?} previousIndex
     * @param {?} width
     * @return {?}
     */
    DataTableComponent.prototype.animateRows = /**
     * @private
     * @param {?} currentIndex
     * @param {?} previousIndex
     * @param {?} width
     * @return {?}
     */
    function (currentIndex, previousIndex, width) {
        this.isMoving = true;
        if (currentIndex > previousIndex) {
            /** @type {?} */
            var totalMove = currentIndex - previousIndex;
            /** @type {?} */
            var previousIndexMove = width * totalMove + 'px';
            /** @type {?} */
            var currentIndexMove = -1 * width + 'px';
            this.Moved[previousIndex] = 'translate3d(' + previousIndexMove + ', 0px, 0px)';
            for (var j = currentIndex; j >= previousIndex; --j) {
                if (j === previousIndex)
                    continue;
                this.Moved[j] = 'translate3d(' + currentIndexMove + ', 0px, 0px)';
            }
        }
        else {
            /** @type {?} */
            var totalMove = previousIndex - currentIndex;
            /** @type {?} */
            var previousIndexMove = width + 'px';
            /** @type {?} */
            var currentIndexMove = -1 * width * totalMove + 'px';
            this.Moved[previousIndex] = 'translate3d(' + currentIndexMove + ', 0px, 0px)';
            for (var j = currentIndex; j <= previousIndex; ++j) {
                if (j === previousIndex)
                    continue;
                this.Moved[j] = 'translate3d(' + previousIndexMove + ', 0px, 0px)';
            }
        }
        this.ref.detectChanges();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.swapped = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var clientWidth = (event.container.element.nativeElement.clientWidth / this.columnDefs.length) - 10;
        this.Moved = [];
        if (this.previousIndex === undefined)
            this.previousIndex = event.previousIndex;
        this.animateRows(event.currentIndex, this.previousIndex, clientWidth);
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.onContextMenuOff = /**
     * @return {?}
     */
    function () {
        this.contextmenu = false;
    };
    /**
     * @param {?} data
     * @return {?}
     */
    DataTableComponent.prototype.deleteData = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        for (var i = 0; i < data.length; ++i) {
            if (data[i] && data[i].length) {
                if (this.TableRows[i].rowSelect)
                    this.rowData[i] = undefined;
            }
        }
        for (var j = 0; j < this.rowData.length; ++j) {
            if (!this.rowData[j]) {
                this.rowData.splice(j, 1);
                j--;
            }
        }
        this.contextMenuData = [];
        this.tableDraw();
    };
    /**
     * @param {?} pasteData
     * @return {?}
     */
    DataTableComponent.prototype.pasteData = /**
     * @param {?} pasteData
     * @return {?}
     */
    function (pasteData) {
        /*
             for (let i = 0; i < this.contextMenuData.length; ++i) {
             if (!this.contextMenuData[i])
             continue;
             let row = this.contextMenuData[i];
             for (let j = 0; j < row.length; ++j) {
             if (!row[j]) {
             continue;
             }
             if (!pasteData[pasteRow][pasteColumn]) {
             pasteColumn++;
             continue;
             }
             this.PagedRows[i].data[j] = pasteData[pasteRow][pasteColumn];
             if (!prevCol) {
             prevCol = j;
             }
             else if (prevCol !== j) {
             pasteColumn++;
             prevCol = j;
             }
             }
             pasteRow++;
             }
             */
        /** @type {?} */
        var startRow;
        /** @type {?} */
        var startColumn;
        for (var i = 0; i < this.contextMenuData.length; ++i) {
            if (!this.contextMenuData[i])
                continue;
            /** @type {?} */
            var row = this.contextMenuData[i];
            for (var j = 0; j < row.length; ++j) {
                if (!row[j]) {
                    continue;
                }
                startRow = i;
                startColumn = j;
            }
        }
        for (var i = 0; i < pasteData.length; ++i) {
            for (var j = 0; j < pasteData[i].length; ++j) {
                /** @type {?} */
                var columnToBeFilled = startColumn + j;
                if (columnToBeFilled >= (this.columnDefs.length)) {
                    continue;
                }
                this.PagedRows[startRow + i].data[columnToBeFilled] = pasteData[i][j];
            }
        }
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.onCtrlV = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var pasteData = this.clipboardService.getClipboardData();
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.onCtrlC = /**
     * @return {?}
     */
    function () {
        this.clipboardService.copyToClipboard(this.contextMenuData);
    };
    /**
     * @param {?} column
     * @param {?} event
     * @return {?}
     */
    DataTableComponent.prototype.toggleFilter = /**
     * @param {?} column
     * @param {?} event
     * @return {?}
     */
    function (column, event) {
        /** @type {?} */
        var positionX = event.clientX + 135;
        if (positionX > window.innerWidth) {
            this.filterMenuX = -220;
        }
        else {
            this.filterMenuX = 25;
        }
        for (var i = 0; i < this.columnDefs.length; ++i) {
            if (i === column)
                continue;
            this.columnDefs[i].showFilter = false;
        }
        this.columnDefs[column].showFilter = !this.columnDefs[column].showFilter;
    };
    /**
     * @param {?} $event
     * @param {?} value
     * @return {?}
     */
    DataTableComponent.prototype.onRowSizeChange = /**
     * @param {?} $event
     * @param {?} value
     * @return {?}
     */
    function ($event, value) {
        this.pageSize = (parseInt(value) > this.rowData.length) ? this.rowData.length : this.rowSizes[0];
        this.tableDraw();
    };
    /**
     * @private
     * @return {?}
     */
    DataTableComponent.prototype.tableDraw = /**
     * @private
     * @return {?}
     */
    function () {
        this.FilterRowCount = this.rowData.length;
        this.TotalRows = this.rowData.length;
        this.FilterData = new Array(this.columnDefs.length);
        this.createTableData();
        this.TotalPages = Math.ceil(this.rowData.length / this.pageSize);
        this.ToRecord = this.pageSize;
    };
    /**
     * @return {?}
     */
    DataTableComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.theme) {
            this.theme = "standard";
        }
        this.commonFilterData = new Array(this.columnDefs.length);
        this.dragTheme = this.theme + '-drag';
        this.pageSize = (this.rowSizes[0] > this.rowData.length) ? this.rowData.length : this.rowSizes[0];
        this.tableDraw();
        this.clipboardService.getPasteEvent().subscribe((/**
         * @param {?} data
         * @return {?}
         */
        function (data) { return _this.pasteData(data); }));
        this.dataTableService.getOnDeleteEvent().subscribe((/**
         * @param {?} data
         * @return {?}
         */
        function (data) { return _this.deleteData(data); }));
    };
    DataTableComponent.decorators = [
        { type: Component, args: [{
                    selector: 'data-grid',
                    template: "<!--\r\n<div class=\"tableConfig\">\r\n  <label for=\"rowCount\">Rows: </label>\r\n  <select id=\"rowCount\" (change) = \"onRowSizeChange( $event, $event.target.value)\">\r\n    <option *ngFor=\"let r of rowSizes\" [value]=\"r\">{{r}}</option>\r\n  </select>\r\n</div>\r\n-->\r\n<div  class=\"table-container\">\r\n  <div *ngIf=\"commonSearch\" class=\"common-search\" [ngClass]=\"theme\">\r\n    <label>\r\n      Search:\r\n      <input type=\"search\" placeholder=\"Search...\" (keyup)=\"onCommonFilter($event.target.value)\"/>\r\n    </label>\r\n  </div>\r\n<table class=\"dataTable\" (click)=\"onContextMenuOff()\" [ngClass]=\"theme\">\r\n  <thead>\r\n  <tr class=\"header\" cdkDropList cdkDropListOrientation=\"horizontal\" (cdkDropListDropped)=\"drop($event)\"\r\n      (cdkDropListSorted)=\"swapped($event)\">\r\n    <th style=\"width: 5px\"></th>\r\n    <th style=\"width: 5px\"></th>\r\n    <th *ngIf=\"rowSelection\" style=\"width: 5px\"></th>\r\n    <th *ngFor=\"let item of columnDefs ; let i = index\" cdkDrag\r\n        [ngStyle]=\"{'width':item.width}\" [ngClass]=\"dragTheme\"\r\n        resizable>\r\n      <div [ngClass]=\"dragTheme\" *cdkDragPreview>\r\n        <div>{{item.headerName}}</div>\r\n      </div>\r\n\r\n      <div class=\"headerText\" cdkDragHandle>\r\n        <i (click)=\"sortColumn(i)\" class=\"fa fa-sort\" *ngIf=\"item.sortState == null && item.sort\"\r\n           aria-hidden=\"true\"> </i>\r\n        <i (click)=\"sortColumn(i)\" class=\"fa fa-sort-asc\" *ngIf=\"item.sortState && item.sort\" aria-hidden=\"true\"> </i>\r\n        <i (click)=\"sortColumn(i)\" class=\"fa fa-sort-desc\"\r\n           *ngIf=\"!item.sortState && !(item.sortState == null) && item.sort\" aria-hidden=\"true\"> </i>\r\n        {{item.headerName}}\r\n      </div>\r\n      <span class=\"col-resize col-resize-cursor \" grabber></span>\r\n    </th>\r\n  </tr>\r\n  <tr class=\"filter\">\r\n    <th style=\"width: 5px\">\r\n      #\r\n    </th>\r\n    <th style=\"width: 5px\">\r\n\r\n    </th>\r\n    <th style=\"width: 5px\" *ngIf=\"rowSelection\">\r\n      <label class=\"container\">\r\n        <input type=\"checkbox\" (change)=\"checkedRowSelection($event, true);\" [(ngModel)]=\"selectAllRows\"/>\r\n        <span class=\"checkmark\"></span>\r\n      </label>\r\n    </th>\r\n    <th *ngFor=\"let item of columnDefs; let i = index\" [ngStyle]=\"{'width':item.width}\">\r\n      <input *ngIf=\"item.filter\" [disabled]=\"item.columnFilter\" (keyup)=\"filter(i,$event.target.value)\" type=\"text\"\r\n             width=\"calc(100% - 23px)\"\r\n             height=\"10\"/>\r\n      <span>\r\n          <i class=\"fa fa-filter filter-icon\" *ngIf=\"item.filter && item.columnFilter\"\r\n             (click)=\"toggleFilter(i, $event)\">\r\n          </i>\r\n        <app-column-filter (clickOutside)=\"item.showFilter=false\" [exclude]=\"'.filter-icon'\"\r\n                           [FilterValues]=\"columnDefs[i].uniqueFilterValues\" [Column]=\"i\"\r\n                           class=\"filterDropdown\"\r\n                           [renderFunction]=\"item.cellRender\"\r\n                           (OnFilterChange)=\"checkedColumnFilter($event)\" *ngIf=\"item.filter && item.columnFilter\"\r\n                           [hidden]=\"!item.showFilter\" [theme]=\"theme\" [ngStyle]=\"{'left.px': filterMenuX}\">\r\n        </app-column-filter>\r\n        </span>\r\n      <span class=\"col-resize\"></span>\r\n\r\n    </th>\r\n  </tr>\r\n  </thead>\r\n  <tbody (click)=\"onContextMenuOff()\" tabindex=\"0\" ctrlKeys\r\n         (ctrlC)=\"onCtrlC()\"\r\n         (ctrlV)=\"onCtrlV()\" cdkDropList (cdkDropListDropped)=\"dropRow($event)\">\r\n  <tr *ngFor=\"let row of PagedRows; let rowCount = index\" [hidden]=\"row.filteredOut\" [@enterAnimation]\r\n      cdkDrag\r\n      [ngClass]=\"{'odd': rowCount%2 !== 0, 'even': rowCount%2===0}\">\r\n    <div [ngClass]=\"dragTheme\" *cdkDragPreview>\r\n      <div [innerHTML]=\"row.cellRender(rowCount, 0,row.data[0], columnDefs)\"></div>\r\n    </div>\r\n    <td style=\"width: 5px\" oncontextmenu=\"return false;\" *ngIf=\"rowSelection\" class=\"checkBoxRow\">\r\n      <app-inline-edit\r\n        [theme]=\"theme\"\r\n        [isSelected]=\"false\"\r\n        [renderer]=\"null\"\r\n        (rightClicked)=\"showContextMenu($event)\"\r\n        (changed)=\"checkedRowSelection($event, false)\"\r\n        [isEditable]=\"false\"\r\n        [columnDefs]=\"columnDefs\" [row]=\"rowCount\"\r\n        [column]=\"-2\" [cellValue]=\"rowCount+1\"\r\n        [cellData]=\"[]\">\r\n\r\n      </app-inline-edit>\r\n    </td>\r\n    <td cdkDragHandle style=\"width: 5px; cursor: grab\" oncontextmenu=\"return false;\" *ngIf=\"rowSelection\"\r\n        class=\"checkBoxRow row-drag\">\r\n      <app-inline-edit\r\n        [theme]=\"theme\"\r\n        [isSelected]=\"false\"\r\n        [renderer]=\"null\"\r\n        (rightClicked)=\"showContextMenu($event)\"\r\n        (changed)=\"checkedRowSelection($event, false)\"\r\n        [isEditable]=\"false\"\r\n        [columnDefs]=\"columnDefs\" [row]=\"rowCount\"\r\n        [column]=\"-3\" [cellValue]=\"\"\r\n        [cellData]=\"[]\">\r\n\r\n      </app-inline-edit>\r\n    </td>\r\n    <td style=\"width: 5px\" oncontextmenu=\"return false;\" *ngIf=\"rowSelection\" class=\"checkBoxRow\">\r\n      <app-inline-edit\r\n        [theme]=\"theme\"\r\n        [isSelected]=\"false\"\r\n        [renderer]=\"null\"\r\n        (rightClicked)=\"showContextMenu($event)\"\r\n        (changed)=\"checkedRowSelection($event, false)\"\r\n        [isEditable]=\"false\"\r\n        [columnDefs]=\"columnDefs\" [row]=\"rowCount\"\r\n        [column]=\"-1\" [cellValue]=\"row.rowSelect\"\r\n        [cellData]=\"[]\">\r\n\r\n      </app-inline-edit>\r\n    </td>\r\n\r\n    <td ctrlkeys *ngFor=\"let cell of row.data; let columnCount = index\"\r\n        (click)=\"onClick(rowCount, columnCount, cell, $event)\"\r\n        (mousedown)=\"onDragStart(rowCount, columnCount,$event)\"\r\n        (mouseover)=\"onSelecting(rowCount, columnCount, cell, $event)\"\r\n        (mouseup)=\"onDragEnd(rowCount, columnCount, $event)\"\r\n        [ngStyle]=\"{'width':columnDefs[columnCount].width, transform:isMoving? Moved[columnCount]:'inherit'}\"\r\n        [ngClass]=\"{'column-moving':(isMoving&&(Moved[columnCount]!==undefined))}\"\r\n        oncontextmenu=\"return false;\">\r\n      <app-inline-edit\r\n        [theme]=\"theme\"\r\n        [isSelected]=\"contextMenuData && contextMenuData[rowCount] && !(contextMenuData[rowCount][columnCount] === undefined)\"\r\n        [renderer]=\"row.cellRender\"\r\n        (rightClicked)=\"showContextMenu($event)\"\r\n        (changed)=\"valueChanged($event)\"\r\n        [isEditable]=\"columnDefs[columnCount].isEdit\"\r\n        [columnDefs]=\"columnDefs\" [row]=\"rowCount\"\r\n        [column]=\"columnCount\" [cellValue]=\"cell\"\r\n        [cellData]=\"columnDefs[columnCount].uniqueFilterValues\">\r\n\r\n      </app-inline-edit>\r\n    </td>\r\n  </tr>\r\n  </tbody>\r\n  <tbody *ngIf=\"FilterRowCount === 0\">\r\n  <tr>\r\n    <td [attr.colspan]=\"columnDefs.length\" style=\"text-align: center\">\r\n      No Data\r\n    </td>\r\n  </tr>\r\n  </tbody>\r\n\r\n  <tfoot *ngIf=\"pagination\">\r\n  <tr>\r\n\r\n    <td [attr.colspan]=\"columnDefs.length+3\">\r\n      <span class=\"rowCount\">\r\n        <label for=\"rowCount\">Rows: </label>\r\n        <select id=\"rowCount\" class=\"page-number-selection\"\r\n                (change)=\"onRowSizeChange( $event, $event.target.value)\">\r\n          <option *ngFor=\"let r of rowSizes\" [value]=\"r\">{{r}}</option>\r\n        </select>\r\n      </span>\r\n      <span class=\"pagination-row\">\r\n        <span>{{FromRecord}}</span>\r\n          to\r\n        <span>{{ToRecord}}</span> of <span>{{TotalRows}}</span>\r\n      </span>\r\n      <span class=\"pagination-page-summary\">\r\n        <button type=\"button\" class=\"pagination btFirst\" [disabled]=\"CurrentPage === 1\" (click)=\"firstPage()\">\r\n          <i class=\"fa fa-angle-double-left\"></i>\r\n        </button>\r\n       <button type=\"button\" class=\"pagination btPrevious\" [disabled]=\"CurrentPage === 1\" (click)=\"previousPage()\">\r\n         <i class=\"fa fa-angle-left\"></i>\r\n       </button>Page <span>{{CurrentPage}}</span> of\r\n        <span ref=\"lbTotal\">{{TotalPages}}</span>\r\n        <button type=\"button\" class=\"pagination btNext\" ref=\"btNext\" [disabled]=\"CurrentPage === TotalPages\"\r\n                (click)=\"nextPage()\">\r\n          <i class=\"fa fa-angle-right\"></i>\r\n        </button>\r\n        <button type=\"button\" class=\"pagination btLast\" ref=\"btLast\" [disabled]=\"CurrentPage === TotalPages\"\r\n                (click)=\"lastPage()\">\r\n          <i class=\"fa fa-angle-double-right\"></i>\r\n        </button>\r\n      </span>\r\n    </td>\r\n  </tr>\r\n  </tfoot>\r\n</table>\r\n</div>\r\n<div *ngIf=\"contextmenu\" (click)=\"onContextMenuOff()\">\r\n  <app-context-menu [x]=\"contextmenuX\" [y]=\"contextmenuY\" [isEdit]=\"contextMenuIsEdit\" [theme]=\"theme\"\r\n                    [contextData]=\"contextMenuData\"\r\n                    [rowData]=\"rowData\"\r\n                    [columnData]=\"columnDefs\"></app-context-menu>\r\n</div>\r\n\r\n",
                    animations: [
                        trigger('enterAnimation', [
                            transition(':enter', [
                                style({ transform: 'translateY(28px)', opacity: 0, height: '28px' }),
                                animate('200ms', style({ transform: 'translateY(0)', opacity: 1, height: '0' }))
                            ]),
                            transition(':enter', [
                                style({ transform: 'translateY(0)', opacity: 0, height: '0' }),
                                animate('200ms', style({ transform: 'translateY(28px)', opacity: 1, height: '28px' }))
                            ])
                        ])
                    ],
                    styles: [".dataTable{min-width:1400px}.common-search{width:247px;position:relative;height:50px}.common-search label{position:absolute;top:5px;padding:10px}.light-theme.common-search{background-color:#fff;color:#000}.dark-theme.common-search{background:#1c1f20;color:rgba(245,245,245,.64)}.dark-theme.common-search input[type=search]{background-color:#262c2e;color:#f5f5f5;border:1px solid #fff}.red-theme.common-search{color:#fff;background:#af0b1c}.metrial-theme.common-search{background-color:#fff;color:rgba(0,0,0,.87)}.table-container{width:100%;overflow-x:auto;margin-bottom:40px;border:1px solid #bdc3c7}.light-theme.dataTable,.red-theme.dataTable{font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-weight:400;font-size:12px;background-color:#fff;color:#000}.light-theme.dataTable .container input[type=checkbox]:checked~.checkmark,.red-theme.dataTable .container input[type=checkbox]:checked~.checkmark{border:1px solid #2196f3;border-radius:2px}.light-theme.dataTable .container .checkmark,.red-theme.dataTable .container .checkmark{background-color:#fff;border:1px solid #2196f3}.light-theme.dataTable tbody tr,.red-theme.dataTable tbody tr{border-bottom:1px solid #d9dcde}.light-theme.dataTable tbody tr:hover,.red-theme.dataTable tbody tr:hover{background-color:#ecf0f1}.light-theme.dataTable thead,.red-theme.dataTable thead{background:#f5f7f7;font-weight:600;font-size:12px;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif}.light-theme.dataTable thead th,.red-theme.dataTable thead th{border:0 solid #bdc3c7;border-bottom:1px solid #bdc3c7;color:rgba(0,0,0,.54)}.light-theme.dataTable thead tr.header,.red-theme.dataTable thead tr.header{border:0 solid #bdc3c7}.light-theme.dataTable tfoot,.red-theme.dataTable tfoot{border-top:1px solid #bdc3c7;color:rgba(0,0,0,.54);font-size:12px;font-weight:400}.light-theme.dataTable tfoot .page-number-selection,.red-theme.dataTable tfoot .page-number-selection{font-size:12px;background:#f9f7f7}.light-theme.dataTable tfoot .page-number-selection:focus,.red-theme.dataTable tfoot .page-number-selection:focus{outline:0}.metrial-theme.dataTable{font-family:Roboto,sans-serif;font-weight:400;font-size:13px;background-color:#fff;color:rgba(0,0,0,.87);border:1px solid #e2e2e2}.metrial-theme.dataTable .container input[type=checkbox]:checked~.checkmark{background-color:#fe5991;border:1px solid #fe5991}.metrial-theme.dataTable .container .checkmark{background-color:#fff;border:1px solid #000}.metrial-theme.dataTable .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #fff;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.metrial-theme.dataTable tbody tr{border:1px solid #e2e2e2}.metrial-theme.dataTable tbody tr:hover{background-color:#fafafa}.metrial-theme.dataTable tbody td{line-height:45px;font-size:13px}.metrial-theme.dataTable thead{font-family:Roboto,sans-serif;font-weight:400;font-size:13px;background:#f5f7f7}.metrial-theme.dataTable thead th{border:0 solid #e2e2e2;border-right:1px solid #e2e2e2;border-bottom:1px solid #e2e2e2;color:rgba(0,0,0,.54);line-height:41px}.metrial-theme.dataTable thead th .col-resize{border:none}.metrial-theme.dataTable thead th:hover{color:rgba(0,0,0,.94);background:#d1d3d3}.metrial-theme.dataTable thead tr.header{border:0 solid #e2e2e2;height:56px}.metrial-theme.dataTable thead tr.filter input[type=text]{height:25px}.metrial-theme.dataTable tfoot{border-top:0;border:1px solid #e2e2e2;color:rgba(0,0,0,.38);font-weight:700;font-size:12px;font-family:Roboto,sans-serif}.metrial-theme.dataTable tfoot tr td{line-height:48px;font-family:Roboto,sans-serif;font-weight:400;font-size:13px}table.dataTable{table-layout:fixed;webkit-touch-callout:none;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;width:100%;text-align:left;border-collapse:collapse}table.dataTable .col-resize{border-right:1px solid rgba(0,0,0,.18);position:absolute!important;right:0;top:0;height:60%;margin:4% 0;width:2px}table.dataTable .col-resize-cursor{cursor:col-resize}table.dataTable th{padding:3px 4px}table.dataTable td{line-height:23px;font-size:12px}table.dataTable tbody:focus{outline:0}table.dataTable tbody .checkBoxRow{text-align:left;line-height:20px;padding-left:12px;padding-right:12px}table.dataTable tbody .checkBoxRow.row-drag{padding-left:6px}table.dataTable tbody tr{border-bottom:1px solid #d9dcde;display:table-row!important}table.dataTable tbody tr.odd{background-color:#fcfdfe}table.dataTable tbody tr.even{background-color:#fff}table.dataTable tbody tr td{text-align:center}table.dataTable tbody tr td:focus{outline:0}table.dataTable tbody tr td ::ng-deep a{color:inherit;font-weight:700;text-decoration:none}table.dataTable thead .headerText{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}table.dataTable thead th{text-align:left;line-height:27px;padding-left:12px;padding-right:12px;font-size:12px;font-weight:600;position:relative}table.dataTable thead th i.fa.fa-sort,table.dataTable thead th i.fa.fa-sort-asc,table.dataTable thead th i.fa.fa-sort-desc{padding-right:10px;cursor:pointer}table.dataTable thead th i.filter-icon{padding-left:10px;cursor:pointer}@media only screen and (max-width:600px){table.dataTable thead th i.filter-icon{padding-left:4px}}table.dataTable thead th:first-child{border-left:none}table.dataTable thead th:last-child{border-right:none}table.dataTable thead tr.header{border-bottom-width:1px}table.dataTable thead tr.filter span{position:relative}table.dataTable thead tr.filter input[type=text]{margin:0;width:calc(100% - 30px);height:15px}table.dataTable thead tr.filter input[type=text]:focus{outline:0}table.dataTable thead tr.filter input[disabled]{background:#ebebeb}table.dataTable thead tr.filter .filterDropdown{position:absolute;min-width:220px;top:-7px;z-index:10}table.dataTable tfoot{align-items:center;height:32px;justify-content:flex-end;padding:0 12px}table.dataTable tfoot tr{text-align:right}table.dataTable tfoot .rowCount{margin-right:20px}table.dataTable tfoot button{background:0 0;border:none;cursor:pointer}table.dataTable tfoot button :focus{border:none}table.dataTable tfoot button:disabled{cursor:default}table.dataTable .checkbox-container{padding-left:5px;padding-right:5px}table.dataTable .options-panel{max-height:100px;min-width:40px;overflow:auto;padding-bottom:5px;padding-top:5px}table.dataTable .container{display:block;position:relative;margin-left:-6px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;line-height:13px}table.dataTable .container input[type=checkbox]{position:relative;opacity:0;cursor:pointer;height:0;width:0}table.dataTable .container input[type=checkbox]:checked~.checkmark:after{display:block}table.dataTable .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #2196f3;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}table.dataTable .container .checkmark{position:absolute!important;top:0;left:0;height:12px;width:12px}table.dataTable .checkmark:after{content:\"\";position:absolute;display:none}.red-theme.dataTable thead{color:#fff;background:#af0b1c}.red-theme.dataTable thead .container input[type=checkbox]:checked~.checkmark{border:1px solid #af0b1c;border-radius:2px}.red-theme.dataTable thead .container .checkmark{background-color:#fff;border:1px solid #af0b1c}.red-theme.dataTable thead .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #af0b1c;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.red-theme.dataTable thead th{color:#fff;border-bottom:1px solid #bdc3c78c}.red-theme.dataTable thead th .col-resize{border-right:1px solid rgba(251,248,248,.45)}.red-theme.dataTable tbody tr.odd{background:#f1f1f1}.cdk-drag-preview{box-sizing:border-box;text-align:left;font-size:12px;font-weight:600;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-style:inherit;line-height:16px;background:#fff;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content;color:rgba(0,0,0,.54);border:1px solid #bdc3c7;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.cdk-drag-preview i{display:none}.dark-theme .cdk-drag-placeholder{background-color:#1c1f20}.red-theme .cdk-drag-placeholder{background-color:rgba(175,11,28,.53)}.cdk-drag-placeholder{background-color:#fff;border:1px solid #bdc3c7!important;display:table-cell}.cdk-drag-preview div::before{content:\"\\f047\";font-family:FontAwesome;font-weight:400;font-style:normal;margin:0 0 0 10px;text-decoration:none;padding-right:5px}.cdk-drag-preview.light-theme-drag{padding:5px 10px;color:rgba(0,0,0,.54);background-color:#fff;overflow:hidden}.cdk-drag-preview.red-theme-drag{padding:5px 10px;color:#fff;background-color:rgba(175,11,28,.8);overflow:hidden}.cdk-drag-preview.dark-theme-drag{background-color:#1c1f20;padding:5px 10px;color:rgba(245,245,245,.64)}.cdk-drag-preview.metrial-theme-drag{background-color:#f9f9f9;padding:15px}.cdk-drag-animating{transition:transform 250ms cubic-bezier(0,0,.2,1);transition:transform 250ms cubic-bezier(0,0,.2,1),-webkit-transform 250ms cubic-bezier(0,0,.2,1)}.cdk-drag{transition:transform .3s;transition:transform .3s,-webkit-transform .3s}.cdk-drop-list-dragging th:not(.cdk-drag-placeholder),.column-moving{transition:transform 250ms cubic-bezier(0,0,.2,1);transition:transform 250ms cubic-bezier(0,0,.2,1),-webkit-transform 250ms cubic-bezier(0,0,.2,1)}", ".dark-theme.dataTable{font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-weight:400;font-size:12px;background-color:#1c1f20;color:rgba(245,245,245,.64);border:1px solid #424242}.dark-theme.dataTable .container input[type=checkbox]:checked~.checkmark{border:1px solid #2196f3;border-radius:2px}.dark-theme.dataTable .container .container:hover input~.checkmark{background-color:#f5f5f5;border:1px solid #fff}.dark-theme.dataTable .container .checkmark{background-color:#262c2e;border:1px solid #f5f7f7}.dark-theme.dataTable input[type=text]{background-color:#262c2e;color:#f5f5f5;border:1px solid #fff}.dark-theme.dataTable input[type=text]:disabled{background-color:rgba(48,46,46,.3)!important;border:1px solid rgba(245,245,245,.38)!important}.dark-theme.dataTable tbody{color:#fff}.dark-theme.dataTable tbody tr{border:1px solid #5c5c5c}.dark-theme.dataTable tbody tr:nth-child(even){background-color:#2d3436}.dark-theme.dataTable tbody tr:nth-child(odd){background-color:#262c2e}.dark-theme.dataTable tbody tr:hover{background-color:#3d4749}.dark-theme.dataTable thead{font-weight:600;font-size:12px;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;background:#1c1f20}.dark-theme.dataTable thead th{border-bottom:1px solid #424242;color:rgba(245,245,245,.64)}.dark-theme.dataTable thead th i.icon-filter{color:#fff}.dark-theme.dataTable thead th .col-resize{border-color:#424242}.dark-theme.dataTable thead tr.header{border:0 solid #424242}.dark-theme.dataTable tfoot{border-top:1px solid #424242}.dark-theme.dataTable tfoot button{color:#f5f5f5}.dark-theme.dataTable tfoot button:disabled{color:rgba(245,245,245,.64)}.dark-theme.dataTable tfoot select{line-height:19px!important;background-color:#262c2e;color:#f5f5f5}"]
                }] }
    ];
    /** @nocollapse */
    DataTableComponent.ctorParameters = function () { return [
        { type: ClipboardService },
        { type: FilterService },
        { type: DataTableUtilsService },
        { type: ChangeDetectorRef }
    ]; };
    DataTableComponent.propDecorators = {
        pagination: [{ type: Input }],
        theme: [{ type: Input }],
        columnDefs: [{ type: Input }],
        rowData: [{ type: Input }],
        rowSelection: [{ type: Input }],
        commonSearch: [{ type: Input }]
    };
    return DataTableComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ContextMenuComponent = /** @class */ (function () {
    function ContextMenuComponent(clipboardService, domSanitizer, dataTableService) {
        this.clipboardService = clipboardService;
        this.domSanitizer = domSanitizer;
        this.dataTableService = dataTableService;
        this._x = 0;
        this._y = 0;
        this.MenuItems = [
            {
                text: 'Copy',
                shortcut: 'Ctrl+C',
                icon: 'fa fa-copy',
                onClick: /**
                 * @param {?} $event
                 * @return {?}
                 */
                function ($event) {
                    this.copyFunction(this.contextData, this.clipboardService);
                }
            },
            {
                text: 'Paste', shortcut: 'Ctrl+V', icon: 'fa fa-paste', onClick: /**
                 * @return {?}
                 */
                function () {
                    /** @type {?} */
                    var pasteData = this.clipboardService.getClipboardData();
                }
            },
            {
                text: 'Delete', shortcut: '', icon: 'fa fa-trash-o', onClick: /**
                 * @return {?}
                 */
                function () {
                    this.onDelete.delete(this.contextData);
                }
            }
            /*,
             {
             text: 'Export', shortcut: ' ', icon: null, onClick() {
             let col = [];
             for (let i = 0; i < this.columnData.length; ++i) {
             col.push(this.columnData[i].field);
             }
             const options = {
             fieldSeparator: ',',
             quoteStrings: '"',
             decimalSeparator: '.',
             showLabels: true,
             showTitle: true,
             title: 'Data',
             useTextFile: false,
             useBom: true,
             headers: col
             };
             const csvExporter = new ExportToCsv(options);
        
             csvExporter.generateCsv(this.rowData);    }
             }*/
        ];
    }
    Object.defineProperty(ContextMenuComponent.prototype, "x", {
        get: /**
         * @return {?}
         */
        function () {
            return this._x;
        },
        set: /**
         * @param {?} mx
         * @return {?}
         */
        function (mx) {
            /** @type {?} */
            var positionX = mx + 144;
            if (positionX > window.innerWidth) {
                this._x = mx - 144;
            }
            else {
                this._x = mx;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ContextMenuComponent.prototype, "y", {
        get: /**
         * @return {?}
         */
        function () {
            return this._y;
        },
        set: /**
         * @param {?} my
         * @return {?}
         */
        function (my) {
            this._y = my;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} text
     * @param {?} clipboardService
     * @return {?}
     */
    ContextMenuComponent.prototype.copyTextToClipboard = /**
     * @param {?} text
     * @param {?} clipboardService
     * @return {?}
     */
    function (text, clipboardService) {
        clipboardService.copyToClipboard(text);
    };
    /**
     * @return {?}
     */
    ContextMenuComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        for (var i = 0; i < this.MenuItems.length; ++i) {
            /** @type {?} */
            var item = this.MenuItems[i];
            if (item.text === 'Paste' && !this.isEdit) {
                item.enabled = true;
                item.clipboardService = this.clipboardService;
            }
            if (item.text === 'Copy') {
                item.enabled = true;
                item.contextData = this.contextData;
                item.copyFunction = this.copyTextToClipboard;
                item.clipboardService = this.clipboardService;
            }
            if (item.text === 'Delete') {
                item.enabled = true;
                item.onDelete = this.dataTableService;
                item.contextData = this.contextData;
            }
            else {
                item.enabled = true;
            }
            if (item.text === 'Export') {
                item.columnData = this.columnData;
                item.rowData = this.rowData;
            }
        }
    };
    ContextMenuComponent.decorators = [
        { type: Component, args: [{
                    selector: 'app-context-menu',
                    template: "<div  #contextMenu class=\"context-menu\" [ngClass]=\"theme\" [ngStyle]=\"{'left.px': x, 'top.px': y}\">\r\n  <div class=\"context-menu-list\">\r\n    <div class=\"context-menu-option\" *ngFor=\"let item of MenuItems\" (click)=\"item.onClick($event, contextData, copyTextToClipboard, clipboardService)\"\r\n         [ngClass]=\"{'context-menu-option-disabled':!item.enabled}\">\r\n      <span class=\"context-menu-option-icon\">\r\n        <span *ngIf=\"item.icon!== null\">\r\n          <i [ngClass]=\"item.icon\"></i>\r\n        </span>\r\n      </span>\r\n      <span class=\"context-menu-option-text\">{{item.text}}</span>\r\n      <span class=\"context-menu-option-shortcut\">{{item.shortcut}}</span>\r\n      <span class=\"context-menu-option-popup-pointer\">&nbsp;</span>\r\n    </div>\r\n  </div>\r\n</div>\r\n",
                    styles: [".light-theme.context-menu,.red-theme.context-menu{background-color:#fff;color:#000;border:1px solid #bdc3c7;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-weight:400;font-size:12px}.light-theme.context-menu .context-menu-list .context-menu-option:hover,.red-theme.context-menu .context-menu-list .context-menu-option:hover{background-color:#ecf0f1}.metrial-theme.context-menu{background-color:#fff;color:rgba(0,0,0,.87);box-shadow:0 3px 1px -2px rgba(0,0,0,.2),0 2px 2px 0 rgba(0,0,0,.14),0 1px 5px 0 rgba(0,0,0,.12);font-family:Roboto,sans-serif;font-weight:400;font-size:13px}.metrial-theme.context-menu .context-menu-list .context-menu-option:hover{background-color:#fafafa}.dark-theme.context-menu{background-color:#2d3436;color:#f5f5f5;border:1px solid #bdc3c7;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-weight:400;font-size:12px}.dark-theme.context-menu .context-menu-list .context-menu-option:hover:not(.context-menu-option-disabled){background-color:#3d4749}.context-menu{border-radius:2px;box-shadow:none;padding:0;z-index:5;max-height:100%;overflow-y:auto;position:absolute;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.context-menu .context-menu-list{cursor:default;margin-bottom:4px;margin-top:4px;width:100%;border-collapse:collapse;display:table}.context-menu .context-menu-list span{display:table-cell;vertical-align:middle}.context-menu .context-menu-list .context-menu-option{line-height:32px;padding-left:8px;padding-right:8px;height:28px;display:table-row}.context-menu .context-menu-list .context-menu-option-disabled{opacity:.5}.context-menu .context-menu-list .context-menu-option-text{white-space:nowrap;padding-left:10px;padding-right:30px}.context-menu .context-menu-list .context-menu-option-icon{padding-left:4px;padding-right:4px}.context-menu .context-menu-list .context-menu-option-icon span{height:16px;line-height:0;margin-top:4px}.context-menu .context-menu-list .context-menu-option-shortcut{padding-left:8px}.context-menu .context-menu-list .context-menu-option-popup-pointer{width:24px}"]
                }] }
    ];
    /** @nocollapse */
    ContextMenuComponent.ctorParameters = function () { return [
        { type: ClipboardService },
        { type: DomSanitizer },
        { type: DataTableUtilsService }
    ]; };
    ContextMenuComponent.propDecorators = {
        x: [{ type: Input }],
        y: [{ type: Input }],
        isEdit: [{ type: Input }],
        contextData: [{ type: Input }],
        theme: [{ type: Input }],
        rowData: [{ type: Input }],
        columnData: [{ type: Input }],
        MenuItems: [{ type: Input }],
        contextMenu: [{ type: ViewChild, args: ['contextMenu',] }]
    };
    return ContextMenuComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InlineEditComponent = /** @class */ (function () {
    function InlineEditComponent(clipboardService) {
        this.clipboardService = clipboardService;
        this.changed = new EventEmitter();
        this.rightClicked = new EventEmitter();
        this.isEdit = false;
        this.isCombobox = false;
        this.changedValue = { column: 0, row: 0, value: null };
        this.contextMenuArgs = { x: 0, y: 0, isEdit: false, column: 0, row: 0, mouseEvent: null };
    }
    /**
     * @param {?} event
     * @return {?}
     */
    InlineEditComponent.prototype.handleEnterKey = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.keyCode === 13) {
            this.isEdit = false;
        }
    };
    /**
     * @return {?}
     */
    InlineEditComponent.prototype.onClick = /**
     * @return {?}
     */
    function () {
        if (this.isEdit) {
            this.isEdit = false;
        }
    };
    /**
     * @return {?}
     */
    InlineEditComponent.prototype.onDoubleClick = /**
     * @return {?}
     */
    function () {
        this.isEdit = true;
        this.vc.nativeElement.focus();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    InlineEditComponent.prototype.onChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.changedValue.value = value;
        this.changedValue.row = this.row;
        this.changedValue.column = this.column;
        this.changed.emit(this.changedValue);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    InlineEditComponent.prototype.onRightClick = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.contextMenuArgs.x = event.clientX;
        this.contextMenuArgs.y = event.clientY;
        this.contextMenuArgs.isEdit = this.isEdit;
        this.contextMenuArgs.row = this.row;
        this.contextMenuArgs.column = this.column;
        this.contextMenuArgs.mouseEvent = event;
        this.rightClicked.emit(this.contextMenuArgs);
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    InlineEditComponent.prototype.onFocusOut = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.isEdit = false;
    };
    /**
     * @return {?}
     */
    InlineEditComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.cellData.length > 0) {
            this.isCombobox = true;
        }
        else {
            this.isCombobox = false;
        }
        this.clipboardService.getCopyEvent().subscribe((/**
         * @param {?} data
         * @return {?}
         */
        function (data) {
            if (_this.isSelected) {
                _this.isCoping = true;
                /** @type {?} */
                var that_1 = _this;
                setTimeout((/**
                 * @return {?}
                 */
                function () {
                    that_1.isCoping = false;
                }), 1000);
            }
        }));
    };
    InlineEditComponent.decorators = [
        { type: Component, args: [{
                    selector: 'app-inline-edit',
                    template: "<div class=\"editConatiner\" [ngClass]=\"theme\" tabindex=\"0\">\r\n  <div *ngIf=\"column === -3\" [ngClass]=\"theme\">\r\n    <i class=\"fa fa-th\"></i>\r\n  </div>\r\n  <div *ngIf=\"column === -2\" [ngClass]=\"theme\" >\r\n    {{cellValue}}\r\n  </div>\r\n  <div *ngIf=\"column === -1\" [ngClass]=\"theme\" >\r\n    <label class=\"container\"  (contextmenu)=\"onRightClick($event)\">\r\n      <input type=\"checkbox\" (change)=\"onChange($event.target.checked)\" [(ngModel)]=\"cellValue\"/>\r\n      <span class=\"checkmark\"></span>\r\n    </label>\r\n  </div>\r\n  <div  *ngIf=\"column >= 0\" [hidden]=\"isEditable\" class=\"cell cell-highlight-animation\"\r\n       [ngClass]=\"{'selected-row':isSelected, 'metrial-theme':theme==='metrial-theme', 'highlight':isCoping}\">\r\n    <div (contextmenu)=\"onRightClick($event)\">\r\n      <span [innerHTML]=\"renderer?renderer(row, column, cellValue, columnDefs):''\"></span>\r\n    </div>\r\n  </div>\r\n  <div  *ngIf=\"column >= 0\" (dblclick)=\"onDoubleClick()\" [hidden]=\"!isEditable\" class=\"cell cell-highlight-animation\"\r\n       [ngClass]=\"{'selected-row':isSelected, 'highlight':isCoping}\">\r\n    <div (contextmenu)=\"onRightClick($event)\">\r\n      <span [hidden]=\"isEdit\" [innerHTML]=\"renderer?renderer(row, column, cellValue, columnDefs):''\"></span>\r\n    <span [hidden]=\"!isEdit\" (focusout)=\"onFocusOut($event)\" #name>\r\n  <select *ngIf=\"isCombobox\" (focusout)=\"onFocusOut($event)\" (change)=\"onChange($event.target.value)\">\r\n    <option *ngFor=\"let item of cellData; let i = index\" [value]=\"item.data[0]\"\r\n            [selected]=\"cellValue === item.data[0]\">{{item.data[0]}}\r\n    </option>\r\n  </select>\r\n  <input #name *ngIf=\"!isCombobox\" type=\"text\" [value]=\"cellValue\" (focusout)=\"onFocusOut($event)\"\r\n         (keydown)=\"handleEnterKey($event)\" (change)=\"onChange($event.target.value)\"/>\r\n</span>\r\n    </div>\r\n  </div>\r\n</div>\r\n",
                    styles: [".fa-th:before{color:#d3d3d3}.container:focus{outline:0}.red-theme .cell.highlight{background-color:#0091ea!important}.red-theme .cell.cell-highlight-animation{transition:background-color 1s}.red-theme .cell.selected-row{background-color:rgba(0,145,234,.3)}.red-theme .container input[type=checkbox]:checked~.checkmark{border:1px solid #af0b1c}.red-theme .container .checkmark{background-color:#fff;border:1px solid #af0b1c}.red-theme .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #af0b1c;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.metrial-theme .container input[type=checkbox]:checked~.checkmark{background-color:#fe5991;border:1px solid #fe5991}.metrial-theme .container .checkmark{background-color:#fff;border:1px solid #000}.metrial-theme .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #fff;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.metrial-theme .cell{border:1px solid transparent;padding-left:12px;padding-right:12px}.metrial-theme .cell input[type=text]{height:38px}.metrial-theme .cell select{height:43px}.metrial-theme .cell.highlight{background-color:#fce4ec!important}.metrial-theme .cell.cell-highlight-animation{transition:background-color 1s}.metrial-theme .cell.selected-row{border:1px solid #3f51b5}.metrial-theme .cell:focus,.metrial-theme .cell:hover{outline:initial}.light-theme .container input[type=checkbox]:checked~.checkmark{border:1px solid #2196f3;border-radius:2px}.light-theme .container .checkmark{background-color:#fff;border:1px solid #2196f3}.light-theme .cell{border:1px solid transparent;padding-left:12px;padding-right:12px}.light-theme .cell input[type=text]{height:19px}.light-theme .cell select{height:24px}.light-theme .cell.highlight{background-color:#0091ea!important}.light-theme .cell.cell-highlight-animation{transition:background-color 1s}.light-theme .cell.selected-row{background-color:rgba(0,145,234,.3)}.dark-theme .container input[type=checkbox]:checked~.checkmark{border:1px solid #2196f3;border-radius:2px}.dark-theme .container .container:hover input~.checkmark{background-color:#f5f5f5}.dark-theme .container .checkmark{background-color:#262c2e;border:1px solid #f5f7f7}.dark-theme .cell{border:1px solid transparent;padding-left:12px;padding-right:12px}.dark-theme .cell input[type=text]{line-height:19px;background-color:#262c2e;color:#f5f5f5;border:1px solid #fff}.dark-theme .cell input[type=text]:disabled{border:1px solid #5c5c5c}.dark-theme .cell select{height:24px;background-color:#262c2e;color:#f5f5f5}.dark-theme .cell.highlight{background-color:#00b0ff!important}.dark-theme .cell.cell-highlight-animation{transition:background-color 1s}.dark-theme .cell.selected-row{background-color:rgba(0,145,234,.3)}.editConatiner .cell input[type=text]{width:100%}.editConatiner .cell select{margin:0;font:inherit;outline:0}.editConatiner:focus{outline:0}.checkbox-container{padding-left:5px;padding-right:5px}.options-panel{max-height:100px;min-width:40px;overflow:auto;padding-bottom:5px;padding-top:5px}.container{display:block;position:relative;margin-left:-6px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;line-height:13px}.container input[type=checkbox]{position:relative;opacity:0;cursor:pointer;height:0;width:0}.container input[type=checkbox]:checked~.checkmark:after{display:block}.container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #2196f3;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.container .checkmark{position:absolute!important;top:0;left:0;height:12px;width:12px}.checkmark:after{content:\"\";position:absolute;display:none}"]
                }] }
    ];
    /** @nocollapse */
    InlineEditComponent.ctorParameters = function () { return [
        { type: ClipboardService }
    ]; };
    InlineEditComponent.propDecorators = {
        row: [{ type: Input }],
        column: [{ type: Input }],
        cellData: [{ type: Input }],
        cellValue: [{ type: Input }],
        columnDefs: [{ type: Input }],
        renderer: [{ type: Input }],
        isEditable: [{ type: Input }],
        isSelected: [{ type: Input }],
        changed: [{ type: Output }],
        rightClicked: [{ type: Output }],
        theme: [{ type: Input }],
        vc: [{ type: ViewChild, args: ['name',] }]
    };
    return InlineEditComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ColumnFilterComponent = /** @class */ (function () {
    function ColumnFilterComponent(filterService) {
        this.filterService = filterService;
        this.FilterValues = [];
        this.OnFilterChange = new EventEmitter();
        this.selectAll = false;
        this.selectOne = false;
        this.filteredData = [];
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    ColumnFilterComponent.prototype.clearAll = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.filterColumn('Select All', false);
        this.OnFilterChange.emit({ filteredData: [], column: this.Column });
    };
    /**
     * @param {?} value
     * @param {?} checkboxValue
     * @return {?}
     */
    ColumnFilterComponent.prototype.filterColumn = /**
     * @param {?} value
     * @param {?} checkboxValue
     * @return {?}
     */
    function (value, checkboxValue) {
        if (value === 'Select All' && checkboxValue) {
            this.filteredData = [];
            this.selectOne = true;
            for (var i = 0; i < this.FilterValues.length; ++i) {
                /** @type {?} */
                var filterValue = this.FilterValues[i];
                filterValue.checked = true;
                this.filteredData.push(filterValue.data[0]);
            }
        }
        else if (value === 'Select All' && !checkboxValue) {
            this.filteredData = [];
            for (var i = 0; i < this.FilterValues.length; ++i) {
                /** @type {?} */
                var filterValue = this.FilterValues[i];
                filterValue.checked = false;
                this.selectAll = false;
            }
            this.selectOne = false;
        }
        else if (checkboxValue) {
            this.filteredData.push(value);
            if (this.filteredData.length === this.FilterValues.length) {
                this.selectAll = true;
            }
        }
        else if (!checkboxValue) {
            /** @type {?} */
            var index = this.filteredData.indexOf(value);
            this.selectAll = false;
            if (index >= 0) {
                this.filteredData.splice(index, 1);
            }
        }
    };
    /**
     * @param {?} value
     * @param {?} event
     * @return {?}
     */
    ColumnFilterComponent.prototype.checkedColumnFilter = /**
     * @param {?} value
     * @param {?} event
     * @return {?}
     */
    function (value, event) {
        this.filterColumn(value, event.target.checked);
        this.OnFilterChange.emit({ filteredData: this.filteredData, column: this.Column });
    };
    /**
     * @param {?} text
     * @return {?}
     */
    ColumnFilterComponent.prototype.search = /**
     * @param {?} text
     * @return {?}
     */
    function (text) {
        /** @type {?} */
        var filterOptions = [{ comparator: StringUtilsService.includes, operator: "or", values: [text] }];
        this.FilterValues = this.filterService.filter(filterOptions, this.FilterValues).tableRows;
    };
    /**
     * @return {?}
     */
    ColumnFilterComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
    };
    ColumnFilterComponent.decorators = [
        { type: Component, args: [{
                    selector: 'app-column-filter',
                    template: "<div class=\"filterPanel\" [ngClass]=\"theme\">\r\n  <div class=\"filterHeader\">\r\n    <div class=\"filter-icon-box\">\r\n      <i class=\"feather icon-filter\"></i>\r\n    </div>\r\n    <div class=\"search-box\">\r\n      <input  placeholder=\"Search...\" type=\"text\" (keyup)=\"search($event.target.value)\">\r\n    </div>\r\n    <div class=\"checkbox-container\">\r\n      <label class=\"container select-all\">Select All\r\n        <input type=\"checkbox\" (change)=\"checkedColumnFilter('Select All', $event);\" [(ngModel)]=\"selectAll\"/>\r\n        <span class=\"checkmark\"></span>\r\n      </label>\r\n    </div>\r\n  </div>\r\n  <div class=\"options-panel\">\r\n    <div class=\"checkbox-container\" *ngFor=\"let sname of FilterValues\" [hidden]=\"sname.filteredOut\">\r\n      <label class=\"container\"><span [innerHTML] = \"renderFunction?renderFunction(0,0,sname.data[0], null):sname.data[0]\"></span>\r\n        <input type=\"checkbox\" [value]=\"sname.data[0]\"\r\n               (change)=\"checkedColumnFilter(sname.data[0], $event)\"\r\n               [(ngModel)]=\"sname.checked\"/>\r\n        <span class=\"checkmark\"></span>\r\n      </label>\r\n    </div>\r\n  </div>\r\n  <div class=\"filterFooter\">\r\n    <button type=\"button\" class=\"btn clear-all\" (click)=\"clearAll($event);\">Clear Filter</button>\r\n  </div>\r\n</div>\r\n\r\n",
                    styles: [".light-theme.filterPanel,.red-theme.filterPanel{background:#fff;border:1px solid #bdc3c7;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;color:#000;font-weight:400}.light-theme.filterPanel .filterHeader,.red-theme.filterPanel .filterHeader{border-bottom:1px solid #bdc3c7}.light-theme.filterPanel .filterFooter,.red-theme.filterPanel .filterFooter{border-top:1px solid #bdc3c7}.light-theme.filterPanel .container:hover input~.checkmark,.red-theme.filterPanel .container:hover input~.checkmark{background-color:inherit}.light-theme.filterPanel .container input[type=checkbox]:checked~.checkmark,.red-theme.filterPanel .container input[type=checkbox]:checked~.checkmark{border-radius:2px;border:1px solid #2196f3}.light-theme.filterPanel .container .checkmark,.red-theme.filterPanel .container .checkmark{background-color:#fff;border:1px solid #2196f3}.red-theme.filterPanel{background:#fff;border:1px solid #bdc3c7;font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;color:#000;font-weight:400}.red-theme.filterPanel .filterHeader{border-bottom:1px solid #bdc3c7}.red-theme.filterPanel .filterFooter{border-top:1px solid #bdc3c7}.red-theme.filterPanel .container:hover input~.checkmark{background-color:inherit}.red-theme.filterPanel .container input[type=checkbox]:checked~.checkmark{border-radius:2px;border:1px solid #af0b1c}.red-theme.filterPanel .container .checkmark{background-color:#fff;border:1px solid #af0b1c}.red-theme.filterPanel .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #af0b1c;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.red-theme.filterPanel .filter-icon-box i{color:#af0b1c}.red-theme.filterPanel .clear-all{border-color:#af0b1c!important;color:#af0b1c!important}.dark-theme.filterPanel{background-color:#2d3436;box-shadow:0 3px 1px -2px rgba(0,0,0,.2),0 2px 2px 0 rgba(0,0,0,.14),0 1px 5px 0 rgba(0,0,0,.12);font-family:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;font-weight:400;color:#f5f5f5;border:1px solid #424242}.dark-theme.filterPanel .filterHeader{border-bottom:1px solid #424242}.dark-theme.filterPanel .filterHeader input[type=text]{background-color:#262c2e;color:#f5f5f5;border:1px solid initial}.dark-theme.filterPanel .filterFooter{border-top:1px solid #424242}.dark-theme.filterPanel .filterFooter .clear-all{background:#424242}.dark-theme.filterPanel .container input[type=checkbox]:checked~.checkmark{border:1px solid #2196f3;border-radius:2px}.dark-theme.filterPanel .container .container:hover input~.checkmark{background-color:#f5f5f5}.dark-theme.filterPanel .container .checkmark{background-color:#262c2e;border:1px solid #fff}.dark-theme.filterPanel .filter-icon-box{background:#1c1f20;border-color:#424242}.dark-theme.filterPanel .filter-icon-box i{background:#2d3436;border-color:#424242}.metrial-theme.filterPanel{background:#f5f5f5;box-shadow:0 3px 1px -2px rgba(0,0,0,.2),0 2px 2px 0 rgba(0,0,0,.14),0 1px 5px 0 rgba(0,0,0,.12);font-family:Roboto,sans-serif;font-weight:400;font-size:13px;color:rgba(0,0,0,.87)}.metrial-theme.filterPanel .filterHeader{border-bottom:1px solid #e2e2e2}.metrial-theme.filterPanel .filterFooter button{background-color:transparent;border:0;color:#3f51b5;font-family:inherit;font-size:12px;margin:0;padding:2px}.metrial-theme.filterPanel .container input[type=checkbox]:checked~.checkmark{background-color:#fe5991;border:1px solid #fe5991}.metrial-theme.filterPanel .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #fff;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}.metrial-theme.filterPanel .container .checkmark{background-color:#fff;border:1px solid #000}div.filterPanel .filter-icon-box{background:#f5f7f7;text-align:center;line-height:2.5;border-bottom:1px solid #bdc3c7}div.filterPanel .filter-icon-box i{color:#2196f3;padding:9px 10px 10px;background:#fff;border-left:1px solid #bdc3c7;border-right:1px solid #bdc3c7}div.filterPanel .search-box{padding-left:5px;padding-right:9px;line-height:35px}div.filterPanel .search-box input[type=text]{width:100%}@media only screen and (max-width:600px){div.filterPanel .search-box input[type=text]{width:95%}}div.filterPanel .search-box input[type=text]::-webkit-input-placeholder{font-size:12px;padding-left:5px}div.filterPanel .search-box input[type=text]::-moz-placeholder{font-size:12px;padding-left:5px}div.filterPanel .search-box input[type=text]:-ms-input-placeholder{font-size:12px;padding-left:5px}div.filterPanel .search-box input[type=text]::-ms-input-placeholder{font-size:12px;padding-left:5px}div.filterPanel .search-box input[type=text]::placeholder{font-size:12px;padding-left:5px}div.filterPanel .checkbox-container{padding-left:5px;padding-right:5px}div.filterPanel .options-panel{max-height:120px;min-width:40px;overflow:auto;padding-bottom:5px;padding-top:5px}div.filterPanel .options-panel ::ng-deep a{color:inherit!important}div.filterPanel .options-panel ::ng-deep a:visited{color:inherit}div.filterPanel .container{display:block;position:relative;padding-left:20px;margin-bottom:12px;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;line-height:13px}div.filterPanel .container.select-all{margin-bottom:5px}div.filterPanel .container input[type=checkbox]{position:absolute;opacity:0;cursor:pointer;height:0;width:0}div.filterPanel .container input[type=checkbox]:checked~.checkmark:after{display:block}div.filterPanel .container .checkmark:after{left:4px;top:1px;width:3px;height:7px;border:solid #2196f3;border-width:0 1px 1px 0;-webkit-transform:rotate(45deg);transform:rotate(45deg)}div.filterPanel .container .checkmark{position:absolute;top:0;left:0;height:12px;width:12px}div.filterPanel .checkmark:after{content:\"\";position:absolute;display:none}div.filterPanel .filterFooter{padding:2px 5px;text-align:right}div.filterPanel .filterFooter .clear-all{color:#2196f3;font-size:12px;background:#fff;border:1px solid;border-radius:5px}"]
                }] }
    ];
    /** @nocollapse */
    ColumnFilterComponent.ctorParameters = function () { return [
        { type: FilterService }
    ]; };
    ColumnFilterComponent.propDecorators = {
        FilterValues: [{ type: Input }],
        Column: [{ type: Input }],
        theme: [{ type: Input }],
        renderFunction: [{ type: Input }],
        OnFilterChange: [{ type: Output }]
    };
    return ColumnFilterComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ResizableDirective = /** @class */ (function () {
    function ResizableDirective(elm) {
        this.elm = elm;
        this._startWidth = 0;
    }
    /**
     * @return {?}
     */
    ResizableDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this._host = this.elm.nativeElement;
    };
    /**
     * @return {?}
     */
    ResizableDirective.prototype.dragStart = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var style$$1 = window.getComputedStyle(this._host, undefined);
        this._startWidth = style$$1.width ? parseInt(style$$1.width, 10) : 0;
    };
    /**
     * @param {?} diff
     * @return {?}
     */
    ResizableDirective.prototype.dragging = /**
     * @param {?} diff
     * @return {?}
     */
    function (diff) {
        this._host.style.width = this._startWidth + diff + 'px';
    };
    /**
     * @return {?}
     */
    ResizableDirective.prototype.dragEnd = /**
     * @return {?}
     */
    function () {
        this._startWidth = 0;
    };
    ResizableDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[resizable]'
                },] }
    ];
    /** @nocollapse */
    ResizableDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    return ResizableDirective;
}());
var GrabberDirective = /** @class */ (function () {
    function GrabberDirective(elm, resizable) {
        var _this = this;
        this.elm = elm;
        this.resizable = resizable;
        this.mousedown = (/**
         * @param {?} e
         * @return {?}
         */
        function (e) {
            _this._startOffsetX = e.clientX;
            document.addEventListener('mousemove', _this._boundDragging);
            document.addEventListener('mouseup', _this._boundDragEnd);
            _this.resizable.dragStart();
        });
        this._startOffsetX = 0;
        this._boundDragging = (/**
         * @param {?} e
         * @return {?}
         */
        function (e) { return _this._dragging(e); });
        this._boundDragEnd = (/**
         * @param {?} e
         * @return {?}
         */
        function (e) { return _this._dragEnd(e); });
    }
    /**
     * @private
     * @param {?} e
     * @return {?}
     */
    GrabberDirective.prototype._dragging = /**
     * @private
     * @param {?} e
     * @return {?}
     */
    function (e) {
        /** @type {?} */
        var diff = e.clientX - this._startOffsetX;
        this.resizable.dragging(diff);
    };
    /**
     * @private
     * @param {?} e
     * @return {?}
     */
    GrabberDirective.prototype._dragEnd = /**
     * @private
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this._startOffsetX = 0;
        document.removeEventListener('mousemove', this._boundDragging);
        document.removeEventListener('mouseup', this._boundDragEnd);
        this.resizable.dragEnd();
    };
    GrabberDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[grabber]',
                },] }
    ];
    /** @nocollapse */
    GrabberDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ResizableDirective, decorators: [{ type: Host }, { type: SkipSelf }] }
    ]; };
    GrabberDirective.propDecorators = {
        mousedown: [{ type: HostListener, args: ['mousedown', ['$event'],] }]
    };
    return GrabberDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AngularOpenDatagridModule = /** @class */ (function () {
    function AngularOpenDatagridModule() {
    }
    AngularOpenDatagridModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        DataTableComponent,
                        ContextMenuComponent,
                        InlineEditComponent,
                        ColumnFilterComponent,
                        ResizableDirective,
                        GrabberDirective
                    ],
                    imports: [
                        BrowserModule,
                        DragDropModule,
                        ClickOutsideModule,
                        FormsModule,
                        BrowserAnimationsModule,
                    ],
                    exports: [DataTableComponent],
                    providers: []
                },] }
    ];
    return AngularOpenDatagridModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { DataTableComponent, AngularOpenDatagridModule, ClipboardService as ɵa, ColumnFilterComponent as ɵf, ContextMenuComponent as ɵd, DataTableUtilsService as ɵc, FilterService as ɵb, InlineEditComponent as ɵe, GrabberDirective as ɵh, ResizableDirective as ɵg };

//# sourceMappingURL=angular-open-datagrid.js.map